// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2009 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;

namespace Smdn.Windows.UserInterfaces.Interop {
  [CLSCompliant(false), Flags]
  public enum SLGP_FLAGS : uint {
    SHORTPATH   = 1,
    UNCPRIORITY = 2,
    RAWPATH     = 4,
  }

  /// <summary>Flags determining how the links with missing targets are resolved.</summary>
  [CLSCompliant(false), Flags]
  public enum SLR_FLAGS : uint {
    /// <summary>
    /// Do not display a dialog box if the link cannot be resolved. 
    /// When SLR_NO_UI is set, a time-out value that specifies the 
    /// maximum amount of time to be spent resolving the link can 
    /// be specified in milliseconds. The function returns if the 
    /// link cannot be resolved within the time-out duration. 
    /// If the timeout is not set, the time-out duration will be 
    /// set to the default value of 3,000 milliseconds (3 seconds). 
    /// </summary>
    NO_UI       = 1,
    /// <summary>
    /// Allow any match during resolution.  Has no effect
    /// on ME/2000 or above, use the other flags instead.
    /// </summary>
    ANY_MATCH   = 2,
    /// <summary>
    /// If the link object has changed, update its path and list 
    /// of identifiers. If SLR_UPDATE is set, you do not need to 
    /// call IPersistFile::IsDirty to determine whether or not 
    /// the link object has changed. 
    /// </summary>
    UPDATE      = 4,
    /// <summary>Do not update the link information.</summary>
    NOUPDATE    = 8,
    /// <summary>Do not execute the search heuristics.</summary>
    NOSEARCH    = 16,
    /// <summary>Do not use distributed link tracking.</summary>
    NOTRACK     = 32,
    /// <summary>
    /// Disable distributed link tracking. By default, 
    /// distributed link tracking tracks removable media 
    /// across multiple devices based on the volume name. 
    /// It also uses the UNC path to track remote file 
    /// systems whose drive letter has changed. Setting 
    /// SLR_NOLINKINFO disables both types of tracking.
    /// </summary>
    NOLINKINFO  = 64,
    /// <summary>Call the Microsoft Windows Installer.</summary>
    INVOKE_MSI  = 128,

    /// <summary>
    /// Not documented in SDK.  Assume same as SLR_NO_UI but 
    /// intended for applications without a hWnd.
    /// </summary>
    UI_WITH_MSG_PUMP = 0x101,
  }
}
