using System;
using System.Net.Mail;
using System.Text;
using System.Windows.Forms;

using Smdn.Formats;

namespace Smdn.Applications.SimpleHalt {
  internal static class ReportMailSender {
    private static readonly string defaultEncodingName = "iso-2022-jp";

    internal static string GetDefaultFromAddress()
    {
      return string.Format("{0}@{1}", Application.ProductName, Environment.MachineName);
    }

    public static void SendTest(bool enableSsl, string reportMailToAddress, string reportMailFromAddress)
    {
      SendMessage(enableSsl, reportMailToAddress, reportMailFromAddress, "送信テスト", delegate(MailMessage message) {
        message.IsBodyHtml = false;
        message.Body = "送信テスト";
        message.BodyEncoding = Encoding.GetEncoding(defaultEncodingName);
      });
    }

    public static void SendReport(HaltOption option, Exception exception)
    {
      var statusText = (exception == null)
        ? string.Format("システムを{0}しました", option.MethodString)
        : string.Format("システムの{0}に失敗しました", option.MethodString);

      SendMessage(option.ReportMailSmtpEnableSsl, option.ReportMailToAddress, option.ReportMailFromAddress, statusText, delegate(MailMessage message) {
        var body = new StringBuilder();

        body.Append(statusText);
        body.Append(Chars.CRLF);

        if (!(option.KillIfHung || option.Immediately)) {
          body.Append("(ほかのアプリケーションにより実行がキャンセルされる場合があります)");
          body.Append(Chars.CRLF);
        }

        body.Append(Chars.CRLF);

        body.AppendFormat("設定内容: {0}", option.GetJoinedCommandLineArgs());
        body.Append(Chars.CRLF);

        if (exception != null) {
          body.Append("エラーの詳細:");
          body.Append(Chars.CRLF);
          body.Append(exception);
          body.Append(Chars.CRLF);
        }

        message.IsBodyHtml = false;
        message.Body = body.ToString();
        message.BodyEncoding = Encoding.GetEncoding(defaultEncodingName);
      });
    }

    private static void SendMessage(bool enableSsl, string to, string from, string subject, Action<MailMessage> editMessage)
    {
      if (string.IsNullOrEmpty(from))
        from = GetDefaultFromAddress();

      var message = new MailMessage(from, to);

      message.DeliveryNotificationOptions = DeliveryNotificationOptions.None;
      message.Subject = TextConvert.ToMimeEncodedString(string.Format("[{0}] {1}", Application.ProductName, subject), MimeEncoding.Base64, Encoding.GetEncoding(defaultEncodingName));

      editMessage(message);

      var client = new SmtpClient();

      client.Timeout = 15 * 1000;
      client.EnableSsl = enableSsl;
      client.Send(message);
    }
  }
}
