using System;
using System.IO;
using System.Reflection;
using System.Windows.Forms;

using Smdn.Windows.UserInterfaces;
using Smdn.Windows.UserInterfaces.Shells;

namespace Smdn.Applications.SimpleHalt {
  public class SimpleHalt {
    [STAThread]
    public static int Main(string[] args)
    {
      Console.CancelKeyPress += new ConsoleCancelEventHandler(delegate { Application.Exit(); });

      Config.Load(Path.GetDirectoryName(Application.ExecutablePath));

      HaltOption option = null;
      bool usage = false;

      try {
        foreach (var arg in args) {
          switch (arg.ToLowerInvariant()) {
            case "/?":
            case "--help":
            case "-h":
            case "/help":
              usage = true;
              break;
          }

          if (usage)
            break;
        }

        if (!usage)
          option = HaltOption.CreateFromArgs(args);
      }
      catch (ArgumentException ex) {
        option = null;
        usage = false;

        Console.Error.WriteLine(ex.Message);
        Console.Error.WriteLine("(/?オプションで使い方を表示します)");
      }

      if (usage) {
        WriteUsage(Console.Error);
      }
      else {
        Application.EnableVisualStyles();
        Application.SetCompatibleTextRenderingDefault(false);
        Application.Run(new SimpleHaltApplicationContext(option));

        Config.Save();
      }

      return 0;
    }

    internal static void WriteUsage(TextWriter writer)
    {
      writer.WriteLine("{0} version {1}", Application.ProductName, Application.ProductVersion);
      writer.WriteLine((Attribute.GetCustomAttribute(Assembly.GetExecutingAssembly(), typeof(AssemblyCopyrightAttribute)) as AssemblyCopyrightAttribute).Copyright);
      writer.WriteLine();

      writer.WriteLine("起動方法:");
      writer.WriteLine("  {0} オプション", Path.GetFileName(Application.ExecutablePath));
      writer.WriteLine();

      HaltOption.WriteUsage(writer);

      writer.WriteLine("その他のオプション:");
      writer.WriteLine("  /?, /help    このメッセージを表示する。");
      writer.WriteLine();
    }

    internal static void CreateShortcut(HaltOption option, string path)
    {
      using (var lnk = new ShellLink()) {
        lnk.Arguments = option.GetJoinedCommandLineArgs();
        lnk.Description = option.GetShortDescriptionString();
        lnk.TargetPath = Application.ExecutablePath;
        lnk.IconLocation = new IconLocation(Application.ExecutablePath, 0);

        lnk.Save(Path.ChangeExtension(path, ShellLink.Extension));
      }
    }
  }
}